/*****************************************************************************
**  This is part of the dxflib library
**  Copyright (C) 2000 Andrew Mustun
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

#include "dl_dxf.h"

#include <ctype.h>


/**
 * Reads the given DXF file into the graphic.
 */
bool
DL_Dxf::in (const char* file, DL_Graphic* graphic)
{
	FILE *fp;
	bool ret=false;
	
	fp = fopen (file, "rt");
	if (fp) {
		while (readDxfGroups (fp, graphic));
		fclose (fp);
		ret=true;
	}
	
	return ret;
}



/**
 * Read groups in a DXF file. This method is recursively called.
 */
bool
DL_Dxf::readDxfGroups (FILE *fp, DL_Graphic* graphic)
{
	static char groupCodeTmp[DL_DXF_MAXLINE];             // Ueeful part of the group code
	static unsigned int groupCode;                        // ... as int
	static char groupValue[DL_DXF_MAXLINE];               // Useful part of the group value
	static char settingValue[DL_DXF_MAXLINE];             // Useful part of the group value if it's a setting
	static int currentEntity = DL_ENTITY_UNKNOWN;
	static char values[DL_DXF_MAXGROUPCODE][DL_DXF_MAXLINE];  // Store the group codes
	static bool firstCall=true;
	static DL_Polyline* currentPolyline = 0;              // Pointer to current polyline to 
	                                                      //   fill in with VERTEX entities
	
	// Init on first call
	if (firstCall) {
		for (int i=0; i<DL_DXF_MAXGROUPCODE; ++i) {
			values[i][0] = '\0';
		}
		firstCall=false;
	}

	// Reset current polyline:
	if (currentEntity!=DL_ENTITY_POLYLINE && currentEntity!=DL_ENTITY_VERTEX) {
		currentPolyline = 0;
	}
	
	// Read one group of the DXF file and chop the lines:
	if (DL_Dxf::getChoppedLine(groupCodeTmp, DL_DXF_MAXLINE, fp) &&
			DL_Dxf::getChoppedLine(groupValue, DL_DXF_MAXLINE, fp) ) {
			
		groupCode = (unsigned int)atoi (groupCodeTmp);
			
		// Proccess the DXF group:
		if (groupCode==0 || groupCode==9) {
			// Add the last read entity 
			switch (currentEntity) {
				case DL_ENTITY_SETTING:
					if (!strcmp(settingValue, "$INSUNITS")) {
						graphic->setUnit (DL_Units::dxfint2unit (atoi (values[70])));
					}
					break;

				case DL_ENTITY_LAYER:
					graphic->addLayer (new DL_Layer (values[2]));
					break;
			
				case DL_ENTITY_POINT: {
					DL_Vector v (atof(values[10]), atof(values[20]));
					graphic->addEntity (new DL_Point (v));
					} break;
					
				case DL_ENTITY_LINE: {
					DL_Vector v1 (atof(values[10]), atof(values[20]), atof(values[30]));
					DL_Vector v2 (atof(values[11]), atof(values[21]), atof(values[31]));
					graphic->addEntity (new DL_Line (v1, v2));
					} break;
					
				case DL_ENTITY_POLYLINE: {
					DL_Vector v (atof(values[10]), atof(values[20]));
					currentPolyline = new DL_Polyline (v);
					graphic->addEntity (currentPolyline);
					} break;

				case DL_ENTITY_VERTEX: {
          DL_Vector v (atof(values[10]), atof(values[20]));
          if (currentPolyline) {
						currentPolyline->addEntity (v);
					}
          } break;
					
				case DL_ENTITY_ARC: {
					DL_Vector v (atof(values[10]), atof(values[20]));
					graphic->addEntity (new DL_Arc (v,
																					atof(values[40]),
																					atof(values[50]),
																					atof(values[51])));
					} break;
					
				case DL_ENTITY_CIRCLE: {
					DL_Vector v (atof(values[10]), atof(values[20]));
					graphic->addEntity (new DL_Circle (v,
																		    		 atof(values[40])));
					} break;
					
				case DL_ENTITY_TEXT: {
					DL_Vector v (atof(values[10]), atof(values[20]));
					graphic->addEntity (new DL_Text (v,
																		    	 values[1],
																					 atof(values[50]),
																					 atof(values[40])));
					} break;
					
				default:
					break;
			}
			currentEntity = DL_ENTITY_UNKNOWN;
			
			// Read DXF settings:
			if (groupValue[0]=='$') {
				if (!strcmp(groupValue, "$INSUNITS")) {
					currentEntity = DL_ENTITY_SETTING;
				}
				strcpy(settingValue, groupValue);
			}
		
			// Read in a new entity:
			else if (!strcmp(groupValue, "LAYER")) {
				currentEntity = DL_ENTITY_LAYER;
			}
			else if (!strcmp(groupValue, "POINT")) {
				currentEntity = DL_ENTITY_POINT;
			}
			else if (!strcmp(groupValue, "LINE")) {
				currentEntity = DL_ENTITY_LINE;
			}
			else if (!strcmp(groupValue, "POLYLINE")) {
				currentEntity = DL_ENTITY_POLYLINE;
			}
			else if (!strcmp(groupValue, "VERTEX")) {
        currentEntity = DL_ENTITY_VERTEX;
      }
			else if (!strcmp(groupValue, "ARC")) {
				currentEntity = DL_ENTITY_ARC;
			}
			else if (!strcmp(groupValue, "CIRCLE")) {
				currentEntity = DL_ENTITY_CIRCLE;
			}
			else if (!strcmp(groupValue, "TEXT")) {
				currentEntity = DL_ENTITY_TEXT;
			}
		}
		
		else {
			// Read in all codes (no matter if needed or not):
			if (groupCode<DL_DXF_MAXGROUPCODE) {
				strncpy (values[groupCode], groupValue, DL_DXF_MAXLINE);
			}
		}
	}
	
	return !feof(fp);
}



/**
 * Reads a line in a file and chopps away the spaces at 
 * start and the linefeed at the end
 */
bool
DL_Dxf::getChoppedLine (char *s, int size, FILE *fp)
{
	if (!feof(fp)) {
		char wholeLine[size];			// the whole line in the file
		char* line;								// Only the useful part of the line
		s[0] = '\0';
				
		line = fgets (wholeLine, size, fp);
		if (!feof(fp) && line) {
			while (line[0]==' ') ++line;
			if (strlen(line)>0) line [strlen(line)-1] = '\0';
			strncpy (s, line, size);
		}
		
		if (strlen(s)>0 && s[strlen(s)-1]==13) s[strlen(s)-1] = '\0';
		
		return true;
	}
	else {
		return false;
	}
}


// EOF
