/*****************************************************************************
**	This is part of the dxflib library
**	Copyright (C) 2000 Andrew Mustun
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License (version 2) as
**	published by the Free Software Foundation.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program; if not, write to the Free Software
**	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
******************************************************************************/

#ifndef DL_FONTS_H
#define DL_FONTS_H

class DL_Entity;

#define DL_FONTS_NUMLETTERS 256	 // number of total letters for one font
#define DL_FONTS_NAMELEN		256	 // max name length for the font name


/** 
 * Structure for one font: 
 */
struct DL_Font {
	DL_Entity* letter[DL_FONTS_NUMLETTERS];  // 256 Pointers to all letters
	int elNum[DL_FONTS_NUMLETTERS];          // 256 values for the number of entities for every letter
	double letWidth[DL_FONTS_NUMLETTERS];    // 256 values for the width of every letter
	char fontName[DL_FONTS_NAMELEN];         // name of this font
	double letterSpacingFactor;              // Factor for Letter Spacing (fontHeight*letterSpacingFactor=letterSpacing)
	double wordSpacingFactor;                // Factor for Word Spacing (fontHeight*wordSpacingFactor=wordSpacing)
	double lineDistanceFactor;               // Factor for Line Distance (fontHeight*lineDistanceFactor=lineDistance)
};


/** 
 * Font handling (single instance) 
 */
class DL_Fonts
{

protected:
	DL_Fonts();
	
public:
	~DL_Fonts();

	/** Initialize the fonts */
	void init();

	/*! Pointer to the one and only DL_Fonts instance */
	static DL_Fonts* getFonts();
	/*! Reset all fonts (set values to zero) This function does NOT delete the letter elements! */
	void reset();
	/*! Get the Font Number from its name */
	int getFontNumber(const char* name);
	/*! Get the Fonts name from its number */
	char* getFontName(int num);
	/*! Get the Fonts letterSpacingFactor from its number */
	double getFontLetterSpacingFactor(int num);
	/*! Get the Fonts wordSpacingFactor from its number */
	double getFontWordSpacingFactor(int num);
	/*! Get the Fonts lineDistanceFactor from its number */
	double getFontLineDistanceFactor(int num);
	/*! Get the total number of fonts found */
	int getNumFonts() { return numFonts; }
	/*!
			Get width of a letter
			\param font font number of the letter
			\param letter ascii code of the letter
			\param height font height
	*/
	double getLetterWidth(int font,
	                      char letter, 
	                      double height);
	/*!
			Get width of a string which can contain line breaks
			\param font Font
			\param text String
			\param height Font height
			\param letSpace Letter spacing
			\param wordSpace Word spacing
	*/
	double getTextWidth(int font,
	                    const char* text,
	                    double height, 
	                    double letSpace, 
	                    double wordSpace);
											
	/*!
			Get width of a line of a string which contains line breaks
			\param font Font
			\param text String
			\param index Line number
			\param height Font height
			\param letSpace Letter spacing
			\param wordSpace Word spacing
			\param fixedWidth Use same width for all letters (as 'W')
	*/
	double getTextLineWidth(int font,
	                        const char* text,
	                        int index,
	                        double height, 
	                        double letSpace, 
	                        double wordSpace,
	                        bool fixedWidth=false);
													
	/*!
			Get max. height of a string which can contain line breaks
			\param font Font
			\param text String
			\param height Font height
			\param lineDist Line distance
	*/
	double getTextHeight(int font,
	                     const char* text,
	                     double height, 
	                     double lineDist);
											
	/*!
			Get a single element from a letter
			\param font	 Font
			\param letter Letter number
			\param height Font height
			\param x			x offset
			\param y			y offset
			\param angle	angle
			\param align	Alignment flags (E_LEFT / E_RIGHT / E_CENTER)
			\param index	Index of the element in the letter
			\param el		 Pointer to the memory where the element gets saved
	*/
	void getLetterElement(int font,
	                      char letter, 
	                      double height,
	                      double x, double y,
	                      double angle,
	                      int align,
 	                      int index,
	                      DL_Entity* el);
	/*!
			Get number of elements in a letter
			\param letter Letter
			\param font	 Font
	*/
	int getLetterElementNumber(int font,
	                           char letter);
	/*!
			Check if a Font number is valid
			\param font The font number
	 */
	bool isFontValid(int font) { if(font>=0 && font<numFonts) return true;
	                             else return false; }
	/*!
			Check if a Letter is available (has at least one element)
			\param let The letter index
	*/
	bool isLetterAvailable(int font, char letter);


private:
	/*! Delete all Fonts */
	void deleteFonts();
	/** Lookup all fonts and store the paths in list fontPathList */
	void lookupFonts( const char* fontPath );
	/*! Read Fonts from all directories	*/
	void readFonts();

protected:
	/*! The one and only instance of DL_Fonts */
	static DL_Fonts* singleInstance;

	/** List of all font paths of all fonts found */
	std::list<const char*> fontPathList;

	/*! The single Fonts */
	CL_Font* font;
	/*! Number of Fonts */
	int		numFonts;			// number of fonts
	

};


#endif

// EOF


